package fr.mael.jiwigo.ui.mainframe;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;

import javax.swing.AbstractListModel;
import javax.swing.ComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fr.mael.jiwigo.transverse.ImagesManagement;
import fr.mael.jiwigo.transverse.util.Messages;

/**
 * 
   Copyright (c) 2010, Mael
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of jiwigo nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL Mael BE LIABLE FOR ANY
   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * @author mael
 *
 * Function that allows the use to choose a privacy level for the pictures
 * he adds
 */
public class DialogPrivacyLevel extends JDialog implements ActionListener {
    /**
     * Button ok
     */
    private JButton buttonOk;

    /**
     * Combo to allow the user to choose the privacy level
     */
    private JComboBox comboPrivacyLevel;

    /**
     * Checkbox to remember the choice
     */
    private JCheckBox checkBoxRemember;

    /**
     * label
     */
    private JLabel labelChoice;

    /**
     * Constructor
     */
    public DialogPrivacyLevel() {
	super(MainFrame.getInstance());
	setModal(true);
	checkBoxRemember = new JCheckBox(Messages.getMessage("dialogPrivacy_remember"));
	labelChoice = new JLabel(Messages.getMessage("dialogPrivacy_choosePrivacy"));
	JPanel panelChoice = new JPanel(new FlowLayout());
	comboPrivacyLevel = new JComboBox(new privacyLevelModel());
	comboPrivacyLevel.setSelectedIndex(0);
	panelChoice.add(comboPrivacyLevel);
	panelChoice.add(checkBoxRemember);
	JPanel panelButton = new JPanel(new FlowLayout());
	buttonOk = new JButton("Ok");
	buttonOk.addActionListener(this);
	panelButton.add(buttonOk);
	this.setLayout(new BorderLayout());
	this.add(panelChoice, BorderLayout.NORTH);
	this.add(panelButton, BorderLayout.CENTER);

	this.pack();
	this.setLocationRelativeTo(null);
	this.setVisible(true);

    }

    @Override
    public void actionPerformed(ActionEvent arg0) {
	if (arg0.getSource().equals(buttonOk)) {
	    ImagesManagement.getInstance().setPrivacyLevel(comboPrivacyLevel.getSelectedIndex() + 1);
	    if (checkBoxRemember.isSelected()) {
		ImagesManagement.getInstance().setRememberPrivacyLevel(true);
	    }
	    this.dispose();
	}

    }

    /**
     * @author mael
     * Internal class that represents the model of the combobox that allows
     * to choose a privacy level
     */
    class privacyLevelModel extends AbstractListModel implements ComboBoxModel {
	HashMap<Integer, String> mapLevels = new HashMap<Integer, String>();

	String[] privacies = new String[5];

	String selection = null;

	public privacyLevelModel() {
	    HashMap<Integer, String> mapLevels = new HashMap<Integer, String>();
	    mapLevels.put(0, "dialogPrivacy_none");
	    mapLevels.put(1, "dialogPrivacy_contacts");
	    mapLevels.put(2, "dialogPrivacy_friends");
	    mapLevels.put(3, "dialogPrivacy_family");
	    mapLevels.put(4, "dialogPrivacy_admin");
	    privacies[0] = Messages.getMessage("dialogPrivacy_none");
	    privacies[1] = Messages.getMessage("dialogPrivacy_contacts") + ", "
		    + Messages.getMessage("dialogPrivacy_friends") + ", " + Messages.getMessage("dialogPrivacy_family")
		    + ", " + Messages.getMessage("dialogPrivacy_admin");
	    privacies[2] = Messages.getMessage("dialogPrivacy_friends") + ", "
		    + Messages.getMessage("dialogPrivacy_family") + ", " + Messages.getMessage("dialogPrivacy_admin");
	    privacies[3] = Messages.getMessage("dialogPrivacy_family") + ", "
		    + Messages.getMessage("dialogPrivacy_admin");
	    privacies[4] = Messages.getMessage("dialogPrivacy_admin");
	}

	public Object getElementAt(int index) {
	    return privacies[index];
	}

	public int getSize() {
	    return privacies.length;
	}

	public void setSelectedItem(Object anItem) {
	    selection = (String) anItem;
	}

	public Object getSelectedItem() {
	    if (selection != null)
		return selection;
	    else
		return null;
	}
    }
}
